/*=============================================================================
	UnCollision.cpp: Higor's collision octree functions
=============================================================================*/


#include "XC_Engine.h"

#ifndef DISABLE_COLLISION
#include "UnXC_Col.h"

static INT DebugCycles		= 0;
static INT RayCycles		= 0;
static INT TraceCycles		= 0;
static INT TotalCycles		= 0;
static INT GridCycles		= 0;
static INT OctreeGridAssignment[TWO_EXP_OGA];

#ifdef KILL_ENGINE
	XC_ENGINE_API FCollisionCacus* CollisionCacus = NULL;
#endif

//We gotta give an index to each piece, num=last in TwoExp
//Move to CPP
static void CalculateGridIdxTable( INT* IT, INT NT)
{
	INT* Start = IT;
	INT j = 1;
	*IT++ = MAX_OCTREE_DEPTH;
	while ( j<OCTREE_GRID_ARRAY )
	{
		INT add = (IT-Start);
		for ( INT i=0 ; i<add ; i++ )
			*IT++ = MAX_OCTREE_DEPTH - j;
		j++;
	}
	check( IT - Start == NT );
}

inline INT GridIdx( FLOAT Line, const FLOAT& Total)
{
	return OctreeGridAssignment[Clamp( appFloor( Line * (FLOAT)TWO_EXP_OGA * _Reciprocal(Total) ), 0, TWO_EXP_OGA-1)];
}

inline bool GEZero( FLOAT value )
{
	const INT value_as_int = *(INT*)&value;
	return ( value_as_int >= 0 );
}

//Nice utilitary function I got from UE sources
//Not as fast as it looks
static UBOOL BoxBoxIntersect(const FBox& box1,const FBox& box2)
{
	if( box1.Min.X > box2.Max.X || box2.Min.X > box1.Max.X )
		return false;
	if( box1.Min.Y > box2.Max.Y || box2.Min.Y > box1.Max.Y )
		return false;
	if( box1.Min.Z > box2.Max.Z || box2.Min.Z > box1.Max.Z )
		return false;
	return true;
}

inline FLOAT Sign( const FLOAT& X)
{
	if ( X > 0 )			return 1.f;
	else if ( X < 0 )		return -1.f;
	return 0.f;
}

inline FLOAT fast_sign_nozero(FLOAT f)
{
    FLOAT r = 1.0f;
    (INT&)r |= ((INT&)f & 0x80000000); // mask sign bit in f, set it in r if necessary
    return r;
}

// returns 1.0f for positive floats, -1.0f for negative floats, 0.0f for zero
inline FLOAT fast_sign(FLOAT f)
{
	if (((INT&)f & 0x7FFFFFFF)==0)
    	return 0.f; // test exponent & mantissa bits: is input zero?
	else
	{
		FLOAT r = 1.0f;
		(INT&)r |= ((INT&)f & 0x80000000); // mask sign bit in f, set it in r if necessary
		return r;
    }
}

inline UBOOL ValidCollidingActor( AActor* Actor)
{	return Actor->bCollideActors && !Actor->bDeleteMe;
}

inline UBOOL BrushMover( AActor *Actor)
{
	return (Actor->Brush && Actor->bIsMover );
}
inline UBOOL UsesMeshPrimitive( AActor *Actor)
{
	return *(((DWORD*)Actor)+408) & 32;
}

/*
XC_Engine: Property bCollideActors: Offset=408, Flags=35, Bitmask=1, Size=4
XC_Engine: Property bCollideWorld: Offset=408, Flags=33, Bitmask=2, Size=4
XC_Engine: Property bBlockActors: Offset=408, Flags=33, Bitmask=4, Size=4
XC_Engine: Property bBlockPlayers: Offset=408, Flags=33, Bitmask=8, Size=4
XC_Engine: Property bProjTarget: Offset=408, Flags=33, Bitmask=16, Size=4
*/


//============================================================================//
//============================================================================//
//===                       FCollisionCacus methods                        ===//
//============================================================================//
//============================================================================//

//Creates a cached static primitive for this actor
void FCollisionCacus::SetPrimitive( AActor *Actor, UPrimitive* BasePrim)
{
	if ( !Actor )
		return;
	if ( !BasePrim ) //Remove primitive data
	{
		if ( UsesMeshPrimitive(Actor) )
			*(((DWORD*)Actor)+408) -= 32;
		return;
	}
}

void FCollisionCacus::SetupGrids()
{
	if ( !GridData )
		GridData = new FGridData();
	INT i;
	for ( i=0 ; i<OCTREE_GRID_ARRAY ; i++ )
	{
		DepthGrid[i].Grid = &(GridData->Grid0);
		DepthGrid[i].Grid += GridData->Position[i];
	}
	DepthGrid[0].SetBaseGrid( this);
	for ( i=1 ; i<OCTREE_GRID_ARRAY ; i++ )
		DepthGrid[i].SetGrid( i, BoundingBox);
	static INT HasGrids = 0;
	if ( !HasGrids++ )
		CalculateGridIdxTable( &OctreeGridAssignment[0], TWO_EXP_OGA);
}

FCheckResult* FCollisionCacus::ActorLineCheck( FMemStack& Mem, FVector End, FVector Start, FVector Extent, BYTE ExtraNodeFlags )
{
	if ( End == Start )
		return ActorPointCheck( Mem, Start, Extent, ExtraNodeFlags);

	guard( FCollisionCacus::ActorLineCheck);
//	clock( RayCycles);
	FPrecomputedRay Ray( End, Start, Extent); //40 ms every 100 frames (Warzone), this is very expensive
//	unclock( RayCycles);
	//FAST QUERY! Get nodes from grid based on Start/End
	FCollisionOctree* StartNode = this;
	INT j = GridIdx( Ray.ExtendedBox.Max.X - Ray.ExtendedBox.Min.X, DepthGrid[0].BoxSize.X); //Insane optimization
	INT XYZ[3];
	DepthGrid[j].GetXYZ( Ray.ExtendedBox.Min, XYZ);
	for ( ; j>0 ; j-- )
	{
		FCollisionOctree* TestNode = DepthGrid[j].GetByXYZ( XYZ[0], XYZ[1], XYZ[2]);
		if ( !TestNode || !FitsBox( Ray.ExtendedBox, TestNode->BoundingBox) )
		{
			XYZ[0] = XYZ[0] >> 1;
			XYZ[1] = XYZ[1] >> 1;
			XYZ[2] = XYZ[2] >> 1;
			continue;
		}
		StartNode = TestNode;
		break;
	}
	FCheckResult* Result = StartNode->FastLineQuery( Mem, End, Start, Extent, ExtraNodeFlags, Ray, NULL);
//	if ( Result && !appStricmp( Result->Actor->GetName(), TEXT("VisibleTeleporter0")) )
//		debugf( NAME_Log, TEXT("Teleporter hit %s: D(%f,%f,%f)"), Result->Actor->GetName(), End.X-Start.X,End.Y-Start.Y,End.Z-Start.Z);
	if ( !Result || !Result->GetNext() )
		return Result;
	INT Found = 0;
	FCheckResult* Hits[128] = {NULL};
	for ( FCheckResult* Link=Result ; Link ; Link=Link->GetNext() )
	{
		Hits[Found++] = Link;
		if ( Found >= 127 ) //Sanity break
			break;
	}
	//Sorting is done by moving elements down
	for ( INT i=1 ; i<Found ; i++ )
		if ( i>0 && (Hits[i]->Time < Hits[i-1]->Time) )
		{
			Exchange( Hits[i], Hits[i-1] );
			if ( i > 1 )
				i -= 2; //Eventually becomes -1
		}
	for ( i=0 ; i<Found ; i++ )
		Hits[i]->Next = Hits[i+1];
	return Hits[0];
	unguard;
}

FCheckResult* FCollisionCacus::ActorPointCheck( FMemStack& Mem, FVector Location, FVector Extent, DWORD ExtraNodeFlags )
{
	guard(FCollisionCacus::ActorPointCheck);

	//FAST QUERY! Get nodes from grid based on Location
	FCollisionOctree* StartNode = this;
	INT XYZ[3];
	DepthGrid[MAX_OCTREE_DEPTH].GetXYZ( Location, XYZ);
	for ( INT j=MAX_OCTREE_DEPTH ; j>0 ; j-- )
	{
		FCollisionOctree* TestNode = DepthGrid[j].GetByXYZ( XYZ[0], XYZ[1], XYZ[2]);
			XYZ[0] = XYZ[0] >> 1;
			XYZ[1] = XYZ[1] >> 1;
			XYZ[2] = XYZ[2] >> 1;
		if ( !TestNode ) //Nothing here
			continue;
		if ( !Extent.IsZero() && !FitsBox( Location, ExpandByExtent(TestNode->BoundingBox, -Extent)) ) //Point not inside reduced box
			continue;
		StartNode = TestNode;
		break;
	}
	return StartNode->FastPointQuery( Mem, Location, Extent, ExtraNodeFlags, NULL);
	unguard;
}

FCheckResult* FCollisionCacus::ActorRadiusCheck( FMemStack& Mem, FVector Location, FLOAT Radius, DWORD ExtraNodeFlags )
{
	return (FCheckResult*) RadiusQuery( Mem, Location, Radius, ExtraNodeFlags, NULL);
}

FCheckResult* FCollisionCacus::ActorEncroachmentCheck( FMemStack& Mem, AActor* Actor, FVector Location, FRotator Rotation, DWORD ExtraNodeFlags )
{
	guard( FCollisionCacus::ActorEncroachmentCheck);
	if ( BrushMover(Actor) )
	{
		Exchange( Actor->Location, Location);
		FBox Box = Actor->Brush->GetCollisionBoundingBox( Actor);

		FCheckResult* Result = MEncroachQuery( Mem, Actor, Box);
		Exchange( Actor->Location, Location);
		return Result;
	}
	else if ( Actor->bBlockPlayers || Actor->bBlockActors ) //Griding did nothing!!! let's stop the CPU usage here
	{
		const FBox ActorBox( Location-Actor->GetCylinderExtent(), Location+Actor->GetCylinderExtent() );

		FCollisionOctree* StartNode = this;
		INT j = GridIdx( ActorBox.Max.X - ActorBox.Min.X, DepthGrid[0].BoxSize.X); //Insane optimization
		INT XYZ[3];
		DepthGrid[j].GetXYZ( Location, XYZ);
		for ( ; j>0 ; j-- )
		{
			FCollisionOctree* TestNode = DepthGrid[j].GetByXYZ( XYZ[0], XYZ[1], XYZ[2]);
			if ( !TestNode || !FitsBox( ActorBox, TestNode->BoundingBox) )
			{
				XYZ[0] = XYZ[0] >> 1;
				XYZ[1] = XYZ[1] >> 1;
				XYZ[2] = XYZ[2] >> 1;
				continue;
			}
			StartNode = TestNode;
			break;
		}
		return StartNode->FastAEncroachQuery( Mem, Actor, Location, ActorBox);
	}
	return NULL;
	unguard;
}



//Quick Add actor/mover
//Also, don't make children nodes if there's 3 or less actors in box
void FCollisionCacus::AddActor( AActor *Actor )
{
	guard( FCollisionCacus::AddActor);
//	check( Actor);
	if ( !ValidCollidingActor(Actor) )
		return;
	guard( InfoConstruct);
	FCacusActorInfo NewInfo( Actor); //Construct info for this actor

//	UBOOL bLog = !appStricmp( Actor->GetName(), TEXT("Mover0") );

	FVector PrimSize = NewInfo.Box.Max - NewInfo.Box.Min;
	//Use the grid no matter what
//	INT j=MAX_OCTREE_DEPTH;
	INT j = GridIdx( PrimSize.X, DepthGrid[0].BoxSize.X); //Insane optimization, eliminates need of X fit check
	INT Max_j = j;
	guard( NodeLoop);
	while ( j>=0 )
	{
		if ( /*(DepthGrid[j].BoxSize.X >= PrimSize.X) &&*/ (DepthGrid[j].BoxSize.Y >= PrimSize.Y) && (DepthGrid[j].BoxSize.Z >= PrimSize.Z) ) //Fits
		{
			INT RawIdx = DepthGrid[j].GetIdx( NewInfo.Location);
			FCollisionOctree* TestNode = DepthGrid[j].Get( RawIdx); //BUGFIX, use center point all the time
			if ( TestNode && FitsBox( NewInfo.Box, TestNode->BoundingBox) )
			{
				while ( j < Max_j )
				{
					if ( TestNode->HitsCenterAxes( NewInfo.Box) )
						break; //Doesn't go in sub-box
					if ( (DepthGrid[j+1].BoxSize.X < PrimSize.X) || (DepthGrid[j+1].BoxSize.Y < PrimSize.Y) || (DepthGrid[j+1].BoxSize.Z < PrimSize.Z) )
						break; //Doesn't fit sub-box
					INT Idx = 0;
					FBox SubBox = TestNode->GetSubOctantBox( NewInfo.Location, Idx);
					if ( !TestNode->Children[Idx] )
					{
						if ( TestNode->Actors.Num() + TestNode->BrushMovers.Num() < MIN_ACTORS_FOR_SUBNODE )
						{
							NewInfo.SubOctant = Idx;
							break; //Don't derivate this actor into a child node, too expensive for now
						}
						TestNode->Children[Idx] =  new FCollisionOctree( TestNode, SubBox, TestNode->CurDepth+1, (BYTE)Idx & 0xFF);
						ChildSet( TestNode->Children[Idx] );
					}
					TestNode = TestNode->Children[Idx];
					j++;
				}
//if ( bLog )
//	debugf( NAME_Log, TEXT("ADDING TO %i WITH DEPTH %i (%i,%i,%i) to (%i,%i,%i)"), TestNode->MIndex, TestNode->CurDepth, (INT)TestNode->BoundingBox.Min.X, (INT)TestNode->BoundingBox.Min.Y, (INT)TestNode->BoundingBox.Min.Z, (INT)TestNode->BoundingBox.Max.X, (INT)TestNode->BoundingBox.Max.Y, (INT)TestNode->BoundingBox.Max.Z  );
				if ( BrushMover( Actor) )
					TestNode->BrushMovers.AddItem( NewInfo);
				else
					TestNode->Actors.AddItem( NewInfo);
				Actor->CollisionTag = TestNode->MIndex;
//	debugf( NAME_Log, TEXT("ADDED %s TO %i WITH DEPTH %i"), Actor->GetName(), TestNode->MIndex, TestNode->CurDepth);
				return;
			}
		}
		j--;
	}
	unguardf( (TEXT("%i"), j ) );
	if ( BrushMover( Actor) )
		BrushMovers.AddItem( NewInfo);
	else
		Actors.AddItem( NewInfo);
	unguardf( (TEXT("%s"), Actor->GetName() ) );
	Actor->CollisionTag = 0;
	Actor->ColLocation = Actor->Location;
//debugf( NAME_Log, TEXT("ADDED %s TO BASE NODE"));
	unguard;
}


//Quick remove actor/mover
//No sanity checks performed for speed reasons
//Does not perform memory deallocations
void FCollisionCacus::RemoveActor( AActor *Actor )
{
	guard( FCollisionCacus::RemoveActor);
	if ( ColList.IsValidIndex(Actor->CollisionTag) && ColList(Actor->CollisionTag) )
	{
		INT i=0;
		TArray<FCacusActorInfo>* AArray = ((TArray<FCacusActorInfo>*) &(ColList(Actor->CollisionTag)->Actors)) + BrushMover( Actor);
		INT* ArrayNum = ((INT*) AArray) + 1;
		while ( i<*ArrayNum )
		{
			if ( (*AArray)(i).Actor == Actor )
			{
				if ( i != (*ArrayNum)-1 )
					(*AArray)(i) = AArray->Last();
#ifdef CACUS_DEBUG
				check( *ArrayNum == AArray->Num() );
#endif
				(*ArrayNum)--;
				return;
			}
			i++;
		}
	}
	unguard;
}

//Does nothing in default hash
void FCollisionCacus::CheckActorNotReferenced( AActor* Actor )
{
}

void FCollisionCacus::Tick()
{
	Cleanup();
	if ( DebugCycles++ > 100 )
	{
//		debugf( NAME_Log, TEXT("Trace: T=%03.1f, R=%03.1f, L=%03.1f"), GSecondsPerCycle*1000 * TotalCycles, GSecondsPerCycle*1000 * RayCycles, GSecondsPerCycle*1000 * TraceCycles);
//		debugf( NAME_Log, TEXT("Cycles measured: %03.1f"), GSecondsPerCycle*1000 * TotalCycles);
		RayCycles = 0;
		TraceCycles = 0;
		TotalCycles = 0;
		DebugCycles = 0;
//		GridCycles = 0;
	}
}

void FCollisionCacus::ChildSet( FCollisionOctree* Child)
{
	Child->Master = this;
	DepthGrid[Child->CurDepth].Set( Child);
	if ( !MinAvailable && ColList(MinAvailable) )
		Child->MIndex = ColList.AddItem( Child);
	else
	{
		Child->MIndex = MinAvailable;
		ColList(MinAvailable++) = Child;
		while ( MinAvailable < ColList.Num() )
		{
			if ( !ColList(MinAvailable) )
				return;
			MinAvailable++;
		}
		MinAvailable = 0;
	}
}

void FCollisionCacus::Exit()
{
	guard( FCollisionCacus::Exit );
	FCollisionOctree::Exit();
	guard( ColList);
	if ( ColList.Num() > 0 )
		ColList.Empty();
	unguard;
	for ( INT i=0 ; i<OCTREE_GRID_ARRAY ; i++ )
		DepthGrid[i].Grid = NULL;
	if ( GridData )
		delete GridData;
	GridData = NULL;
	unguard;
}


FCollisionCacus::~FCollisionCacus()
{
	guard( ~FCollisionCacus );
	Exit();
	CollisionCacus = NULL;
	unguard;
}

//============================================================================//
//============================================================================//
//===                       FCollisionOctree methods                       ===//
//============================================================================//
//============================================================================//

//XC_GameEngine interface
FCacusActorInfo* FCollisionOctree::FindActor( FName ByName)
{
	guard( FCollisionOctree::FindActor);
	for ( INT i=0 ; i<Actors.Num() ; i++ )
	{
		if ( Actors(i).Actor->GetFName() == ByName )
			return &Actors(i);
	}
	for ( i=0 ; i<BrushMovers.Num() ; i++ )
	{
		if ( BrushMovers(i).Actor->GetFName() == ByName )
			return &BrushMovers(i);
	}
	FCacusActorInfo* Result = NULL;
	for ( i=0 ; i<ChildCount ; i++ )
	{
		Result = ReducedChildren[i]->FindActor( ByName);
		if ( Result )
			break;
	}
	return Result;
	unguard;
}

void FCollisionOctree::Cleanup()
{
	guard( FCollisionOctree::Cleanup);
	INT i=0;
	//Cleanup child nodes first
	guard( CleanupChild);
	for ( i=0 ; i<ChildCount ; i++ )
	{
		ReducedChildren[i]->Cleanup();
		if ( ReducedChildren[i]->ChildCount || ReducedChildren[i]->Actors.Num() || ReducedChildren[i]->BrushMovers.Num() )
			continue;
		if ( --(ReducedChildren[i]->CleanupTimer) <= 0 )
		{
			ReducedChildren[i]->Exit();
			delete ReducedChildren[i]; //Causes ChildCount--
			ReducedChildren[i] = ReducedChildren[ChildCount];
			ReducedChildren[ChildCount] = NULL;
		}
	}
	unguard;

	//Move my actors to parent node if we're holding too few actors, parent's Cleanup will delete me later
	guard( ParentCL);
	if ( !ChildCount && CurDepth )
	{
		if ( (Actors.Num() + BrushMovers.Num() + Parent->Actors.Num() + Parent->BrushMovers.Num()) < MIN_ACTORS_FOR_SUBNODE )
		{
			if ( --CleanupTimer <= 1 )
			{
				INT j = 0;
				if ( Actors.Num() )
				{
					j = Parent->Actors.Num();
					Parent->Actors.Add( Actors.Num() );
					appMemcpy( &Parent->Actors(j), &Actors(0), sizeof(struct FCacusActorInfo) * Actors.Num() );
					for ( i=0 ; i<Actors.Num() ; i++ )
					{
						Parent->Actors(i+j).SubOctant = ChildIndex;
						Parent->Actors(i+j).Actor->CollisionTag = Parent->MIndex;
					}
					Actors.Empty();
				}
				if ( BrushMovers.Num() )
				{
					j = Parent->BrushMovers.Num();
					Parent->BrushMovers.Add( BrushMovers.Num() );
					appMemcpy( &Parent->BrushMovers(j), &BrushMovers(0), sizeof(struct FCacusActorInfo) * BrushMovers.Num() );
					for ( i=0 ; i<BrushMovers.Num() ; i++ )
					{
						Parent->BrushMovers(i+j).SubOctant = ChildIndex;
						Parent->BrushMovers(i+j).Actor->CollisionTag = Parent->MIndex;
					}
					BrushMovers.Empty();
				}
			}
		}
		return;
	}
	unguard;

#ifdef CACUS_DEBUG
	guard( CleanupActors);
#endif
	i=0;
	while ( i<Actors.Num() )
	{
		if ( !ValidCollidingActor( Actors(i).Actor) )
		{
			Actors.Remove( i);
			continue;
		}
		if ( Actors(i).SubOctant >= 0 && Children[Actors(i).SubOctant] )
		{//Send actors from this node into existing child
			FCollisionOctree* Child = Children[Actors(i).SubOctant];
			INT j = Child->Actors.AddItem( Actors(i) ); //Move to Child
			Actors.Remove( i);
			if ( Child->HitsCenterAxes( Child->Actors(j).Box) )
				Child->Actors(j).SubOctant = -1;
			else
				Child->Actors(j).SubOctant = Child->GetSubOctantIdx( Child->Actors(j).Location);
			Child->Actors(j).Actor->CollisionTag = Child->MIndex;
			continue;
		}
		i++;
	}
	i=0;
	while ( i<BrushMovers.Num() )
	{
		if ( !ValidCollidingActor( BrushMovers(i).Actor) )
		{
			BrushMovers.Remove( i);
			continue;
		}
		if ( BrushMovers(i).SubOctant >= 0 && Children[BrushMovers(i).SubOctant] )
		{//Send movers from this node into existing child
			FCollisionOctree* Child = Children[BrushMovers(i).SubOctant];
			INT j = Child->BrushMovers.AddItem( BrushMovers(i) ); //Move to Child
			BrushMovers.Remove( i);
			if ( Child->HitsCenterAxes( Child->BrushMovers(j).Box) )
				Child->BrushMovers(j).SubOctant = -1;
			else
				Child->BrushMovers(j).SubOctant = Child->GetSubOctantIdx( Child->BrushMovers(j).Location);
			Child->BrushMovers(j).Actor->CollisionTag = Child->MIndex;
			continue;
		}
		i++;
	}
#ifdef CACUS_DEBUG
	unguard;
#endif

	unguard;
}

//Recursive actor iteration
//Obtain actors in a specific radius
FIteratorActorList* FCollisionOctree::RadiusQuery( FMemStack& Mem, const FVector& Location, const FLOAT& Radius, const DWORD& ExtraNodeFlags, FIteratorActorList* PrevLink )
{
	guard( FCollisionOctree::RadiusQuery);
	FLOAT RSq = Radius * Radius;
	if ( (CurDepth > 0) && (!FitsBox( Location, BoundingBox.ExpandBy( Radius))
		|| (FDistSquared(CenterPoint,Location) > RSq + FDistSquared(CenterPoint,BoundingBox.Max))) )
		return PrevLink;
	for ( INT i=0 ; i<Actors.Num() ; i++ )
		if ( ValidCollidingActor(Actors(i).Actor) && (FDistSquared( Actors(i).Location, Location) <= RSq) )
			PrevLink = new(Mem)FIteratorActorList( PrevLink, Actors(i).Actor);
	for ( i=0 ; i<BrushMovers.Num() ; i++ )
		if ( ValidCollidingActor(BrushMovers(i).Actor) && (FDistSquared( BrushMovers(i).Location, Location) <= RSq) )
			PrevLink = new(Mem)FIteratorActorList( PrevLink, BrushMovers(i).Actor);
	for ( i=0 ; i<ChildCount ; i++ )
		PrevLink = ReducedChildren[i]->RadiusQuery( Mem, Location, Radius, ExtraNodeFlags, PrevLink);
	return PrevLink;
	unguardf( (TEXT("Depth %i"), CurDepth) );
}


//Recursive actor iteration
//Obtains actors touching a volume/point
FCheckResult* FCollisionOctree::PointQuery( FMemStack& Mem, const FVector& Location, const FVector& Extent, const DWORD& ExtraNodeFlags, FCheckResult* PrevLink)
{
	guard( Deep);
	if ( CurDepth > 0 && !FitsBox( Location, ExpandByExtent(BoundingBox, Extent)) )
		return PrevLink;
	for ( INT i=0 ; i<Actors.Num() ; i++ )
	{
		if ( !ValidCollidingActor(Actors(i).Actor) )
			continue;
		FVector Diff = Actors(i).Location - Location;
		if ( (Diff.Size2D() <= Actors(i).Actor->CollisionRadius + Extent.X)
			&& (Abs(Diff.Z) <= Actors(i).Actor->CollisionHeight + Extent.Z) )
		{
			guard( NewLink);
			PrevLink = new(Mem)FCheckResult( 1.f, PrevLink);
			PrevLink->Actor = Actors(i).Actor;
			PrevLink->Location = Actors(i).Actor->Location;
			if ( !(PrevLink->Location - Location).IsNearlyZero() )
				PrevLink->Normal = _UnsafeNormal( Location - PrevLink->Location);
			unguard;
		}
	}
	for ( i=0 ; i<BrushMovers.Num() ; i++ )
	{
		if ( !ValidCollidingActor(BrushMovers(i).Actor) || !BrushMovers(i).Actor->Brush )
			continue;
		FBox MoverBox = BrushMovers(i).Box;
//		FBox MoverBox = BrushMovers(i)->Brush->GetCollisionBoundingBox(BrushMovers(i));
		if ( !Extent.IsZero() )
			MoverBox = ExpandByExtent( MoverBox, Extent);
		if ( FitsBox( Location, MoverBox ) )
		{
			FCheckResult Hit(1.f, PrevLink);
			if ( !BrushMovers(i).Actor->Brush->PointCheck( Hit, BrushMovers(i).Actor, Location, Extent, ExtraNodeFlags) )
			{
				PrevLink = new(Mem)FCheckResult(1.f,PrevLink);
				appMemcpy( PrevLink, &Hit, sizeof( Hit) );
			}
		}
		continue;
	}
	for ( i=0 ; i<ChildCount ; i++ )
		PrevLink = ReducedChildren[i]->PointQuery( Mem, Location, Extent, ExtraNodeFlags, PrevLink);
	return PrevLink;
	unguardf( (TEXT("Depth %i"), CurDepth) );
}


//Semi-recursive actor iteration
//Obtains actors touching a volume/point
FCheckResult* FCollisionOctree::FastPointQuery( FMemStack& Mem, const FVector& Location, const FVector& Extent, const DWORD& ExtraNodeFlags, FCheckResult* PrevLink)
{
	guard( FCollisionOctree::FastPointQuery);
	INT i;
	for ( FCollisionOctree* Link=this ; Link ; Link=Link->Parent )
	{	if ( Link->Actors.Num() )
		{
			for ( i=0 ; i<Link->Actors.Num() ; i++ )
			{
				if ( !ValidCollidingActor(Link->Actors(i).Actor) )
					continue;
				FVector Diff = Link->Actors(i).Location - Location;
				if ( (Diff.SizeSquared2D() <= Square(Link->Actors(i).Actor->CollisionRadius + Extent.X))
					&& (Abs(Diff.Z) <= Link->Actors(i).Actor->CollisionHeight + Extent.Z) )
				{
					PrevLink = new(Mem)FCheckResult( 1.f, PrevLink);
					PrevLink->Actor = Link->Actors(i).Actor;
					PrevLink->Location = Link->Actors(i).Location;
					if ( !(PrevLink->Location - Location).IsNearlyZero() )
						PrevLink->Normal = _UnsafeNormal( Location - PrevLink->Location);
				}
			}
		}
		if ( Link->BrushMovers.Num() )
		{
			for ( i=0 ; i<Link->BrushMovers.Num() ; i++ )
			{
				if ( !ValidCollidingActor(Link->BrushMovers(i).Actor) || !Link->BrushMovers(i).Actor->Brush )
					continue;
				FBox MoverBox = Link->BrushMovers(i).Box;
				if ( !Extent.IsZero() )
					MoverBox = ExpandByExtent( MoverBox, Extent);
				if ( FitsBox( Location, MoverBox ) )
				{
					FCheckResult Hit(1.f, PrevLink);
					if ( !Link->BrushMovers(i).Actor->Brush->PointCheck( Hit, Link->BrushMovers(i).Actor, Location, Extent, ExtraNodeFlags) )
					{
						PrevLink = new(Mem)FCheckResult(1.f,PrevLink);
						appMemcpy( PrevLink, &Hit, sizeof( Hit) );
					}
				}
			}
		}
	}
	for ( i=0 ; i<ChildCount ; i++ )
		PrevLink = ReducedChildren[i]->PointQuery( Mem, Location, Extent, ExtraNodeFlags, PrevLink);
	return PrevLink;
	unguard;
}

//Recursive actor iteration
//Obtains actors touching a brush
FCheckResult* FCollisionOctree::MEncroachQuery( FMemStack& Mem, AActor* Actor, const FBox& MoverBox, FCheckResult* PrevLink)
{
#ifdef CACUS_DEBUG
	guard( Deep);
#endif
	for ( INT i=0 ; i<Actors.Num() ; i++ )
	{	//this bCollideWorld reject is responsible for a massive speedup!
		if ( !Actors(i).Actor->bCollideWorld || !Actors(i).Actor->bBlockActors || !BoxBoxIntersect( MoverBox, Actors(i).Box) ) //Quick box rejects
			continue;
		AActor* TestActor = Actors(i).Actor;
//		if ( !ValidCollidingActor(TestActor) ) //I assume Cleanup handles this
//			continue;
		FCheckResult TestHit(1.0);
		if ( Actor->Brush->PointCheck( TestHit, Actor, Actors(i).Location, TestActor->GetCylinderExtent(), 0 )==0 )
		{
			TestHit.Actor = TestActor;
			TestHit.Primitive = NULL;
			TestHit.Next = PrevLink;
			PrevLink = new(GMem)FCheckResult;
			*PrevLink = TestHit;
		}
	}
	for ( i=0 ; i<ChildCount ; i++ )
		if ( BoxBoxIntersect( MoverBox, ReducedChildren[i]->BoundingBox) )
			PrevLink = ReducedChildren[i]->MEncroachQuery( Mem, Actor, MoverBox, PrevLink);
	return PrevLink;
#ifdef CACUS_DEBUG
	unguardf( (TEXT("Depth %i"), CurDepth) );
#endif
}

//Recursive actor iteration
//Obtains actors touching an actor (future loc)
FCheckResult* FCollisionOctree::AEncroachQuery( FMemStack& Mem, AActor* Actor, const FVector& Location, const FBox& ActorBox, FCheckResult* PrevLink)
{
	for ( INT i=0 ; i<Actors.Num() ; i++ )
	{
		if ( !BoxBoxIntersect( ActorBox, Actors(i).Box) ) //Quick box rejects, we don't need to perform Z check later
			continue;
		AActor* TestActor = Actors(i).Actor;
//		if ( !ValidCollidingActor(TestActor) ) //I assume Cleanup handles this
//			continue;
		if ( (FVector(Location.X-Actors(i).Location.X,Location.Y-Actors(i).Location.Y,0.f).SizeSquared2D() < Square( Actor->CollisionRadius+TestActor->CollisionRadius))  )
		{
			PrevLink = new(Mem)FCheckResult( 0.f, PrevLink);
			PrevLink->Actor = TestActor;
			PrevLink->Location = Actors(i).Location;
		}
	}
	for ( i=0 ; i<ChildCount ; i++ )
		if ( BoxBoxIntersect( ActorBox, ReducedChildren[i]->BoundingBox) )
			PrevLink = ReducedChildren[i]->AEncroachQuery( Mem, Actor, Location, ActorBox, PrevLink);
	return PrevLink;
}

//Recursive actor iteration
//Obtains actors touching an actor (future loc)
FCheckResult* FCollisionOctree::FastAEncroachQuery( FMemStack& Mem, AActor* Actor, const FVector& Location, const FBox& ActorBox, FCheckResult* PrevLink)
{
	INT i;
	for ( FCollisionOctree* Link=this ; Link ; Link=Link->Parent )
	{
		for ( i=0 ; i<Link->Actors.Num() ; i++ )
		{
			if ( !BoxBoxIntersect( ActorBox, Link->Actors(i).Box) ) //Quick box rejects, we don't need to perform Z check later
				continue;
			AActor* TestActor = Link->Actors(i).Actor;
			if ( (FVector(Location.X-TestActor->Location.X,Location.Y-TestActor->Location.Y,0.f).SizeSquared2D() < Square( Actor->CollisionRadius+TestActor->CollisionRadius))  )
			{
				PrevLink = new(Mem)FCheckResult( 0.f, PrevLink);
				PrevLink->Actor = TestActor;
				PrevLink->Location = TestActor->Location;
			}
		}
	}
	for ( i=0 ; i<ChildCount ; i++ )
		if ( BoxBoxIntersect( ActorBox, ReducedChildren[i]->BoundingBox) )
			PrevLink = ReducedChildren[i]->AEncroachQuery( Mem, Actor, Location, ActorBox, PrevLink);
	return PrevLink;
}


//Recursive actor iteration
//Obtains actors touching a line
FCheckResult* FCollisionOctree::LineQuery
(
 FMemStack& Mem
, const FVector& End
, const FVector& Start
, const FVector& Extent
, const BYTE& ExtraNodeFlags
, FPrecomputedRay& Ray
,  FCheckResult* PrevLink
)
{
	guard( Deep);
	//CurDepth is always zon-zero after the FastLineQuery implementation
	if ( /*Ray.PreBoxCheck(CenterPoint, BoundingBox) &&*/ Ray.IntersectsBox(BoundingBox) )
	{
		INT i;
		if ( Actors.Num() )
		{
			FVector HitLocation(0,0,0);
			FVector HitNormal(0,0,0);
			for ( i=0 ; i<Actors.Num() ; i++ )
			{
//				if ( !BoxBoxIntersect( Actors(i).Box, Ray.ExtendedBox) ) //NEGLIGIBLE
//					continue;
				if ( !Ray.HitsGCylActor(Actors(i).Actor, &HitLocation, &HitNormal) )
					continue;
				PrevLink = new(Mem) FCheckResult( Clamp( (HitLocation-Start).Size() / Ray.Dist, 0.f, 1.f) , PrevLink);
				PrevLink->Actor = Actors(i).Actor;
				PrevLink->Location = HitLocation;
				PrevLink->Normal = HitNormal;
			}
		}
		if ( BrushMovers.Num() )
		{
			for ( i=0 ; i<BrushMovers.Num() ; i++ )
			{
				if ( BrushMovers(i).Actor->Brush && Ray.IntersectsBox( BrushMovers(i).Box) )
				{
					FCheckResult Hit(0);
					if ( !BrushMovers(i).Actor->Brush->LineCheck( Hit, BrushMovers(i).Actor, End, Start, Extent, ExtraNodeFlags) )
					{
						FCheckResult* NewLink = new(Mem)FCheckResult(Hit);
						NewLink->GetNext() = PrevLink;
						PrevLink = NewLink;
					}
				}
			}
		}
		for ( i=0 ; i<ChildCount ; i++ )
			PrevLink = ReducedChildren[i]->LineQuery( Mem, End, Start, Extent, ExtraNodeFlags, Ray, PrevLink);
	}
	return PrevLink;
	unguard;
}

//Semi-recursive actor iteration
//Obtains actors touching a line
FCheckResult* FCollisionOctree::FastLineQuery
(
 FMemStack& Mem
, const FVector& End
, const FVector& Start
, const FVector& Extent
, const BYTE& ExtraNodeFlags
, FPrecomputedRay& Ray
,  FCheckResult* PrevLink
)
{
	guard( FCollisionOctree::FastLineQuery);
	//Count this node as well
	INT i;
	for ( FCollisionOctree* Link=this ; Link ; Link=Link->Parent )
	{
		if ( Link->Actors.Num() )
		{
			for ( i=0 ; i<Link->Actors.Num() ; i++ )
			{
//				if ( !ValidCollidingActor(Link->Actors(i).Actor) ) //ASSUMING CLEANUP FIXED THIS
//					continue;
//				if ( !BoxBoxIntersect( Link->Actors(i).Box, Ray.ExtendedBox) ) //NEGLIGIBLE
//					continue;
				FVector HitLocation(0,0,0);
				FVector HitNormal(0,0,0);
				if ( !Ray.HitsGCylActor( Link->Actors(i).Actor, &HitLocation, &HitNormal) )
					continue;
				PrevLink = new(Mem) FCheckResult( Clamp( (HitLocation-Ray.Org).Size() / Ray.Dist, 0.f, 1.f) , PrevLink);
				PrevLink->Actor = Link->Actors(i).Actor;
				PrevLink->Location = HitLocation;
				PrevLink->Normal = HitNormal;
			}
		}
		if ( Link->BrushMovers.Num() )
		{
			for ( i=0 ; i<Link->BrushMovers.Num() ; i++ )
			{
//				if ( !ValidCollidingActor(Link->BrushMovers(i).Actor) )
//					continue;
				if ( Link->BrushMovers(i).Actor->Brush && Ray.IntersectsBox( Link->BrushMovers(i).Box) )
				{
					FCheckResult Hit(0);
					if ( !Link->BrushMovers(i).Actor->Brush->LineCheck( Hit, Link->BrushMovers(i).Actor, End, Start, Extent, ExtraNodeFlags) )
					{
						FCheckResult* NewLink = new(Mem)FCheckResult(Hit);
						NewLink->GetNext() = PrevLink;
						PrevLink = NewLink;
					}
				}
			}
		}
	}
	for ( i=0 ; i<ChildCount ; i++ )
		PrevLink = ReducedChildren[i]->LineQuery( Mem, End, Start, Extent, ExtraNodeFlags, Ray, PrevLink);

	return PrevLink;
	unguard;
}


// Obtain a child box based on a position
// This doesn't do any kind of sanity checks for speed reasons
FBox FCollisionOctree::GetSubOctantBox( const FVector& Point, INT& Idx)
{
	Idx = GetSubOctantIdx( Point);
	switch ( Idx )
	{
		case 0:
			return FBox( BoundingBox.Min, CenterPoint);
		case 1:
			return FBox( FVector(BoundingBox.Min.X, BoundingBox.Min.Y, CenterPoint.Z), FVector( CenterPoint.X, CenterPoint.Y, BoundingBox.Max.Z) );
		case 2:
			return FBox( FVector(BoundingBox.Min.X, CenterPoint.Y, BoundingBox.Min.Z), FVector( CenterPoint.X, BoundingBox.Max.Y, CenterPoint.Z) );
		case 3:
			return FBox( FVector(BoundingBox.Min.X, CenterPoint.Y, CenterPoint.Z), FVector( CenterPoint.X, BoundingBox.Max.Y, BoundingBox.Max.Z) );
		case 4:
			return FBox( FVector( CenterPoint.X, BoundingBox.Min.Y, BoundingBox.Min.Z), FVector( BoundingBox.Max.X, CenterPoint.Y, CenterPoint.Z) );
		case 5:
			return FBox( FVector( CenterPoint.X, BoundingBox.Min.Y, CenterPoint.Z), FVector( BoundingBox.Max.X, CenterPoint.Y, BoundingBox.Max.Z) );
		case 6:
			return FBox( FVector( CenterPoint.X, CenterPoint.Y, BoundingBox.Min.Z), FVector( BoundingBox.Max.X, BoundingBox.Max.Y, CenterPoint.Z) );
		case 7:
		default:
			return FBox( CenterPoint, BoundingBox.Max);
	}
/*	if ( Point.X >= CenterPoint.X )
	{	if ( Point.Y >= CenterPoint.Y )
		{	if ( Point.Z >= CenterPoint.Z )
			{
				Idx = 7;
				return FBox( CenterPoint, BoundingBox.Max); //7 = (1,1,1)
			}
			Idx = 6;
			return FBox( FVector( CenterPoint.X, CenterPoint.Y, BoundingBox.Min.Z), FVector( BoundingBox.Max.X, BoundingBox.Max.Y, CenterPoint.Z) ); //6 = (1,1,0)
		}
		if ( Point.Z >= CenterPoint.Z )
		{
			Idx = 5;
			return FBox( FVector( CenterPoint.X, BoundingBox.Min.Y, CenterPoint.Z), FVector( BoundingBox.Max.X, CenterPoint.Y, BoundingBox.Max.Z) ); //5 = (1,0,1)
		}
		Idx = 4;
		return FBox( FVector( CenterPoint.X, BoundingBox.Min.Y, BoundingBox.Min.Z), FVector( BoundingBox.Max.X, CenterPoint.Y, CenterPoint.Z) );//4 = (1,0,0)
	}
	if ( Point.Y >= CenterPoint.Y )
	{	if ( Point.Z >= CenterPoint.Z )
		{
			Idx = 3;
			return FBox( FVector(BoundingBox.Min.X, CenterPoint.Y, CenterPoint.Z), FVector( CenterPoint.X, BoundingBox.Max.Y, BoundingBox.Max.Z) ); //3 = (0,1,1)
		}
		Idx = 2;
		return FBox( FVector(BoundingBox.Min.X, CenterPoint.Y, BoundingBox.Min.Z), FVector( CenterPoint.X, BoundingBox.Max.Y, CenterPoint.Z) ); //2 = (0,1,0)
	}
	if ( Point.Z >= CenterPoint.Z )
	{
		Idx = 1;
		return FBox( FVector(BoundingBox.Min.X, BoundingBox.Min.Y, CenterPoint.Z), FVector( CenterPoint.X, CenterPoint.Y, BoundingBox.Max.Z) ); //1 = (0,0,1)
	}
	Idx = 0;
	return FBox( BoundingBox.Min, CenterPoint);//0 = (0,0,0)*/
}

inline INT FCollisionOctree::GetSubOctantIdx( const FVector& Point)
{
	return ((Point.X >= CenterPoint.X) << 2) | ((Point.Y >= CenterPoint.Y) << 1) | (Point.Z >= CenterPoint.Z);
}

//Manual cleanup, needed in v451
void FCollisionOctree::Exit()
{
	guard( FCollisionOctree::Exit );
	if ( Actors.Num() > 0 )
		Actors.Empty();
	if ( BrushMovers.Num() > 0 )
		BrushMovers.Empty();
	appMemzero( &Actors, 24);
	unguard;
}

FCollisionOctree::~FCollisionOctree()
{
	guard( Destructor);
	INT i;
	if ( Parent )
	{
#ifdef CACUS_DEBUG
		check( Parent->Children[ ChildIndex] == this);
#endif
		Parent->Children[ChildIndex] = NULL;
		Parent->ChildCount--;
	}
	if ( Master )
	{
		if ( CurDepth )
			Master->DepthGrid[CurDepth].Remove( this);
		if ( (MIndex < Master->ColList.Num()) && (Master->ColList(MIndex) == this) )
		{
			Master->ColList(MIndex) = NULL;
			if ( !Master->MinAvailable )
				Master->MinAvailable = MIndex;
			else
				Master->MinAvailable = Min( Master->MinAvailable, MIndex);
		}
	}
	for ( i=0 ; i<8 ; i++ )
		if ( Children[i] )
		{
			Children[i]->Exit();
			delete Children[i]; //Deletion becomes recursive
		}
	unguard;
}


//============================================================================//
//============================================================================//
//===                       FPrecomputedRay methods                        ===//
//============================================================================//
//============================================================================//

FPrecomputedRay::FPrecomputedRay( const FVector& A, const FVector& B, const FVector& TExtent)
	:	End( A)
	,	Org( B)
	,	Dir( A-B)
	,	Extent( &TExtent)
{
	TwoPointBox( ExtendedBox, A, B);
	Dist = Dir.Size();
	if ( !TExtent.IsZero() )
	{
		ExtendedBox.Min -= TExtent;
		ExtendedBox.Max += TExtent;
	}
	if ( Abs(Dir.X) > SMALL_NUMBER || Abs(Dir.Y) > SMALL_NUMBER )
	{
		XAxis = _UnsafeNormal2D( Dir.X, Dir.Y);
		YAxis = FVector( -XAxis.Y, XAxis.X, 0.f); //Rotate 90 to right in 2D
		TraceFunc = &FPrecomputedRay::HitsCylActor;
	}
	else
	{
		appMemzero( &XAxis, 24);
		TraceFunc = &FPrecomputedRay::HitsVCylActor;
	}
	Dir = _UnsafeNormal( Dir);
	//Don't use Inv on zero components
	Inv.X = Dir.X == 0.f ? 0.f : _Reciprocal( Dir.X);
	Inv.Y = Dir.Y == 0.f ? 0.f : _Reciprocal( Dir.Y);
	Inv.Z = Dir.Z == 0.f ? 0.f : _Reciprocal( Dir.Z);
}

FPrecomputedRay::FPrecomputedRay( const FPrecomputedRay& ParentRay, const FLocalBoxIntersectInfo& LocalInfo)
	:	Org( LocalInfo.Start)
	,	End( LocalInfo.End)
	,	Dir( ParentRay.Dir)
	,	Inv( ParentRay.Inv)
	,	XAxis( ParentRay.XAxis)
	,	YAxis( ParentRay.YAxis)
	,	Extent( ParentRay.Extent)
	,	Dist( ParentRay.Dist) //Do i need this?
	,	TraceFunc( ParentRay.TraceFunc)
{
	TwoPointBox( ExtendedBox, End, Org);
	if ( !Extent->IsZero() )
	{
		ExtendedBox.Min -= *Extent;
		ExtendedBox.Max += *Extent;
	}
}


inline UBOOL FPrecomputedRay::HitsGCylActor( AActor* Actor, FVector* HitLocation, FVector* HitNormal)
{
/*		clock( TraceCycles);
		UBOOL Result = (*this.*TraceFunc)( Actor, HitLocation, HitNormal);
		unclock( TraceCycles);
		return Result;*/
	return (*this.*TraceFunc)( Actor, HitLocation, HitNormal);
}

UBOOL FPrecomputedRay::HitsVCylActor( AActor* Actor, FVector* HitLocation, FVector* HitNormal)
{
	if ( Square(Actor->Location.X - Org.X) + Square(Actor->Location.Y - Org.Y) > Square(Extent->X + Actor->CollisionRadius) )
		return 0; //Out of cylinder radius
	FLOAT AdjZ = Actor->Location.Z - Org.Z;
	FLOAT DiffZ = End.Z - Org.Z;
	if ( (AdjZ >= 0 && DiffZ <= 0) || (AdjZ <= 0 && DiffZ >= 0) )
		return 0; //Opposite directions
	if ( Abs(AdjZ)  < (Actor->CollisionHeight + Extent->Z) )
		return 0; //Start point already colliding
	if ( Abs(AdjZ) - (Actor->CollisionHeight + Extent->Z) > Abs(DiffZ) )
		return 0; //No reach
	*HitNormal = FVector(0,0,-fast_sign_nozero(DiffZ));
	*HitLocation = Org;
	(*HitLocation).Z = Actor->Location.Z - (Actor->CollisionHeight + Extent->Z) * fast_sign_nozero(AdjZ);
	return 1;
}

UBOOL FPrecomputedRay::HitsCylActor( AActor* Actor, FVector* HitLocation, FVector* HitNormal)
{
	//Screw quick (X,Y,Z) rejects, I made the octree for this reason!!!
	FVector AdjustedActor = TransformPointByXY( FCoords(Org, XAxis, YAxis, FVector(0,0,1.f)), Actor->Location ); //Gotta optimize that third coordinate
	FLOAT TotalSide = Extent->X + Actor->CollisionRadius;
	if ( Abs(AdjustedActor.Y) > TotalSide )
		return 0; //Out of line's X,Y trajectory
	if ( AdjustedActor.X  < -TotalSide )
		return 0; //Way behind the trace's extent, will never hit
	if ( AdjustedActor.X > (End.X-Org.X) * XAxis.X + (End.Y-Org.Y) * XAxis.Y + TotalSide ) 
		return 0; //Actor further than our trace | another DOT product optimized

	FLOAT TotalHeight = Extent->Z + Actor->CollisionHeight;
	if ( Abs(AdjustedActor.Z) > Abs(End.Z-Org.Z) + TotalHeight )
		return 0; //Actor further than Z trace

	//Trace starts inside actor
	if ( (AdjustedActor.X  < TotalSide) && (Abs(AdjustedActor.Z) < TotalHeight) && (AdjustedActor.SizeSquared2D() < Square(TotalSide))  )
	{
		if ( AdjustedActor.X <= 0.001 )
			return 0; //Tracing away from actor
		*HitLocation = Org;
		if ( Abs(AdjustedActor.Z * 0.99) > TotalHeight ) //Up or down the cylinder
			*HitNormal = FVector(0,0,-fast_sign_nozero(AdjustedActor.Z));
		else //On side or inside
		{
			*HitNormal = _UnsafeNormal2D( Org.X - Actor->Location.X, Org.Y - Actor->Location.Y );
			*HitLocation = *HitLocation + XAxis + *HitNormal; //I have to correct this
//			debugf( NAME_Log, TEXT("Normal #1: %f %f %f, %f"), HitNormal->X, HitNormal->Y, HitNormal->Z, HitNormal->Size() );
		}
		return 1;
	}
	FLOAT XDelta = appSqrt(TotalSide*TotalSide - AdjustedActor.Y*AdjustedActor.Y);

	FVector VStart(0,0,0);
	if ( AdjustedActor.X >= XDelta ) //We can move forward and position ourselves next to the cylinder (side by side)
	{
		VStart = Dir * _appInvSqrt( Dir.X*Dir.X + Dir.Y*Dir.Y ) * (AdjustedActor.X - XDelta);
		if ( Abs(VStart.Z - AdjustedActor.Z) < TotalHeight )
		{
			*HitLocation = VStart + Org;
			*HitNormal = _UnsafeNormal2D( HitLocation->X - Actor->Location.X, HitLocation->Y - Actor->Location.Y);
			return 1;
		}
		if ( Abs(VStart.Z) > Abs(AdjustedActor.Z) ) //We passed the actor already!
			return 0;
	}
	FLOAT ZDif = End.Z-Org.Z;
	if ( (AdjustedActor.Z) * ZDif <= 0 ) 
		return 0; //Different Z directions, never gonna hit

	FLOAT PlaneZ = AdjustedActor.Z - fast_sign_nozero(AdjustedActor.Z) * TotalHeight;
	VStart = Org + (End-Org) * (PlaneZ * _Reciprocal(ZDif));
	if ( (VStart - Actor->Location ).SizeSquared2D() > TotalSide*TotalSide )
		return 0;
	*HitLocation = VStart;
	*HitNormal = FVector(0,0,-fast_sign_nozero(ZDif));
	return 1;
}

INT FPrecomputedRay::PreBoxCheck( const FVector& CenterPoint, const FBox& Box)
{
	FVector AdjustedCenter = TransformPointByXY( FCoords(Org, XAxis, YAxis, FVector(0,0,1.f)), CenterPoint );
//Take this from grid later
	FLOAT TotalSide = (CenterPoint.X-Box.Min.X) + (CenterPoint.Y-Box.Min.Y);
	//Screw quick (X,Y,Z) rejects, I made the octree for this reason!!!
	if ( Abs(AdjustedCenter.Y) > TotalSide )
		return 0; //Out of line's X,Y trajectory
	if ( AdjustedCenter.X  < -TotalSide )
		return 0; //Way behind the trace's extent, will never hit
	if ( AdjustedCenter.X > (End.X-Org.X) * XAxis.X + (End.Y-Org.Y) * XAxis.Y + TotalSide ) 
		return 0; //Actor further than our trace | another DOT product optimized

	FLOAT TotalHeight = Extent->Z + CenterPoint.Z - Box.Min.Z;
	if ( Abs(AdjustedCenter.Z) > Abs(End.Z-Org.Z) + TotalHeight )
		return 0; //Actor further than Z trace

	//Trace starts inside actor
	if ( (AdjustedCenter.X  < TotalSide) && (Abs(AdjustedCenter.Z) < TotalHeight) && (AdjustedCenter.SizeSquared2D() < Square(TotalSide))  )
		return 1;

	FLOAT XDelta = appSqrt(TotalSide*TotalSide - AdjustedCenter.Y*AdjustedCenter.Y);

	FVector VStart(0,0,0);
	if ( AdjustedCenter.X >= XDelta ) //We can move forward and position ourselves next to the cylinder (side by side)
	{
		VStart = Dir * _appInvSqrt( Dir.X*Dir.X + Dir.Y*Dir.Y ) * (AdjustedCenter.X - XDelta);
		if ( Abs(VStart.Z - AdjustedCenter.Z) < TotalHeight )
			return 1;
		if ( Abs(VStart.Z) > Abs(AdjustedCenter.Z) ) //We passed the actor already!
			return 0;
	}
	FLOAT ZDif = End.Z-Org.Z;
	if ( (AdjustedCenter.Z) * ZDif <= 0 ) 
		return 0; //Different Z directions, never gonna hit

	FLOAT PlaneZ = AdjustedCenter.Z - fast_sign_nozero(AdjustedCenter.Z) * TotalHeight;
	VStart = Org + (End-Org) * (PlaneZ * _Reciprocal(ZDif));
	if ( (VStart - CenterPoint ).SizeSquared2D() > TotalSide*TotalSide )
		return 0;
	return 1;

}

INT FPrecomputedRay::IntersectsBox( const FBox& Box)
{
	// Quick rejects, optimized for movement code.
	if( Box.Min.X > ExtendedBox.Max.X || ExtendedBox.Min.X > Box.Max.X
	||	Box.Min.Y > ExtendedBox.Max.Y || ExtendedBox.Min.Y > Box.Max.Y
	||	Box.Min.Z > ExtendedBox.Max.Z || ExtendedBox.Min.Z > Box.Max.Z )
	{
		return 0;
	}

//	FBox EBox( Box.Min-*Extent, Box.Max+*Extent);
	FBox EBox( Box.Min, Box.Max);
	if ( !Extent->IsZero() )
	{
		EBox.Min -= *Extent;
		EBox.Max += *Extent;
	}
	if ( FitsBox( Org, EBox) ) //This algorithm fails when traces start inside the box, let's cover this case
		return 1;

	//Back face culling
	//Checking on both inverses prevents early 'Dir' allocation in FPU registers
	FVector T;
	T.X = ((GEZero(Inv.X) ? EBox.Min.X : EBox.Max.X) - Org.X) * Inv.X;
	T.Y = ((GEZero(Inv.Y) ? EBox.Min.Y : EBox.Max.Y) - Org.Y) * Inv.Y;
	T.Z = ((GEZero(Inv.Z) ? EBox.Min.Z : EBox.Max.Z) - Org.Z) * Inv.Z;

	if ( (T.X >= T.Y) && (T.X >= T.Z) ) //X is maximum
	{
		if ( GEZero(T.X) /*&& (T.X <= Dist) */)
		{
			FLOAT ptY = Org.Y + Dir.Y * T.X;
			FLOAT ptZ = Org.Z + Dir.Z * T.X;
			return ptY >= EBox.Min.Y && ptY <= EBox.Max.Y
				&& ptZ >= EBox.Min.Z && ptZ <= EBox.Max.Z;
		}
	}
	else if ( T.Y >= T.Z ) //Y is maximum
	{
		if ( GEZero(T.Y) /*&& (T.Y <= Dist)*/ )
		{
			FLOAT ptX = Org.X + Dir.X * T.Y;
			FLOAT ptZ = Org.Z + Dir.Z * T.Y;
			return ptX >= EBox.Min.X && ptX <= EBox.Max.X
				&& ptZ >= EBox.Min.Z && ptZ <= EBox.Max.Z;
		}
	}
	else //Z is maximum
	{
		if ( GEZero(T.Z) /*&& (T.Z <= Dist)*/ )
		{
			FLOAT ptX = Org.X + Dir.X * T.Z;
			FLOAT ptY = Org.Y + Dir.Y * T.Z;
			return ptX >= EBox.Min.X && ptX <= EBox.Max.X
				&& ptY >= EBox.Min.Y && ptY <= EBox.Max.Y;
		}
	}
	return 0;
}



//Gives a full new box info to play with
//The implementation is VERY expensive
INT FPrecomputedRay::IntersectsBox( const FBox& Box, FLocalBoxIntersectInfo& Hit)
{
	// Quick rejects, optimized for movement code.
	if( Box.Min.X > ExtendedBox.Max.X || ExtendedBox.Min.X > Box.Max.X
	||	Box.Min.Y > ExtendedBox.Max.Y || ExtendedBox.Min.Y > Box.Max.Y
	||	Box.Min.Z > ExtendedBox.Max.Z || ExtendedBox.Min.Z > Box.Max.Z )
	{
		return 0;
	}

	FBox EBox( Box.Min-*Extent, Box.Max+*Extent);
	FVector T;
	INT HitAxis = 0;
	if ( FitsBox( Org, EBox) ) //This algorithm fails when traces start inside the box, let's cover this case
	{
		Hit.Start = Org;
		goto FRONT_FACE_CULLING;
	}

	//Back face culling
	//Checking on both inverses prevents early 'Dir' allocation in FPU registers
	T.X = ((Inv.X > 0 ? EBox.Min.X : EBox.Max.X) - Org.X) * Inv.X;
	T.Y = ((Inv.Y > 0 ? EBox.Min.Y : EBox.Max.Y) - Org.Y) * Inv.Y;
	T.Z = ((Inv.Z > 0 ? EBox.Min.Z : EBox.Max.Z) - Org.Z) * Inv.Z;

	if ( (T.X >= T.Y) && (T.X >= T.Z) ) //X is maximum
	{
		HitAxis = 0;
		if ( (T.X >= 0) && (T.X <= Dist) )
		{
			FLOAT ptY = Org.Y + Dir.Y * T.X;
			FLOAT ptZ = Org.Z + Dir.Z * T.X;
			if( ptY >= EBox.Min.Y && ptY <= EBox.Max.Y && ptZ >= EBox.Min.Z && ptZ <= EBox.Max.Z )
			{
				Hit.Start.X = Org.X + Dir.X * T.X;
				Hit.Start.Y = ptY;
				Hit.Start.Z = ptZ;
			}
			else
				return 0;
		}
	}
	else if ( T.Y >= T.Z ) //Y is maximum
	{
		HitAxis = 1;
		if ( (T.Y >= 0) && (T.Y <= Dist) )
		{
			FLOAT ptX = Org.X + Dir.X * T.Y;
			FLOAT ptZ = Org.Z + Dir.Z * T.Y;
			if ( ptX >= EBox.Min.X && ptX <= EBox.Max.X && ptZ >= EBox.Min.Z && ptZ <= EBox.Max.Z )
			{
				Hit.Start.X = ptX;
				Hit.Start.Y = Org.Y + Dir.Y *  T.Y;
				Hit.Start.Z = ptZ;
			}
			else
				return 0;
		}
	}
	else //Z is maximum
	{
		HitAxis = 2;
		if ( (T.Z >= 0) && (T.Z <= Dist) )
		{
			FLOAT ptX = Org.X + Dir.X * T.Z;
			FLOAT ptY = Org.Y + Dir.Y * T.Z;
			if ( ptX >= EBox.Min.X && ptX <= EBox.Max.X && ptY >= EBox.Min.Y && ptY <= EBox.Max.Y )
			{
				Hit.Start.X = ptX;
				Hit.Start.Y = ptY;
				Hit.Start.Z = Org.Z + Dir.Z * T.Z;
			}
			else
				return 0;
		}
	}
	Hit.Start += Dir; //Truncate hit a bit, makes future ops faster
	
FRONT_FACE_CULLING:
	Hit.End = End;
	if ( FitsBox( End, EBox) )
		return 1;

	//Front face culling
	//Checking on both inverses prevents early 'Dir' allocation in FPU registers
	T.X = (End.X - (Inv.X > 0 ? EBox.Max.X : EBox.Min.X)) * Inv.X;
	T.Y = (End.Y - (Inv.Y > 0 ? EBox.Max.Y : EBox.Min.Y)) * Inv.Y;
	T.Z = (End.Z - (Inv.Z > 0 ? EBox.Max.Z : EBox.Min.Z)) * Inv.Z;

	if ( (T.X >= T.Y) && (T.X >= T.Z) ) //X is maximum
	{
		if ( (T.X >= 0) && (T.X <= Dist) )
		{
			FLOAT ptY = End.Y - Dir.Y * T.X;
			FLOAT ptZ = End.Z - Dir.Z * T.X;

			Hit.End.X = End.X - Dir.X * T.X;
			Hit.End.Y = ptY;
			Hit.End.Z = ptZ;
		}
	}
	else if ( T.Y >= T.Z ) //Y is maximum
	{
		if ( (T.Y >= 0) && (T.Y <= Dist) )
		{
			FLOAT ptX = End.X - Dir.X * T.Y;
			FLOAT ptZ = End.Z - Dir.Z * T.Y;

			Hit.End.X = ptX;
			Hit.End.Y = End.Y - Dir.Y * T.Y;
			Hit.End.Z = ptZ;
		}
	}
	else //Z is maximum
	{
		if ( (T.Z >= 0) && (T.Z <= Dist) )
		{
			FLOAT ptX = End.X - Dir.X * T.Z;
			FLOAT ptY = End.Y - Dir.Y * T.Z;

			Hit.End.X = ptX;
			Hit.End.Y = ptY;
			Hit.End.Z = End.Z - Dir.Z * T.Z;
		}
	}
	Hit.End -= Dir; //Truncate hit a bit, makes future ops faster
	return 1;
}

//TODO: MOVE TO ASSEMBLER
void FPrecomputedRay::TwoPointBox( FBox& Box, const FVector& A, const FVector& B)
{
	if ( A.X > B.X )
	{
		Box.Min.X = B.X;
		Box.Max.X = A.X;
	}
	else
	{
		Box.Min.X = A.X;
		Box.Max.X = B.X;
	}
	if ( A.Y > B.Y )
	{
		Box.Min.Y = B.Y;
		Box.Max.Y = A.Y;
	}
	else
	{
		Box.Min.Y = A.Y;
		Box.Max.Y = B.Y;
	}
	if ( A.Z > B.Z )
	{
		Box.Min.Z = B.Z;
		Box.Max.Z = A.Z;
	}
	else
	{
		Box.Min.Z = A.Z;
		Box.Max.Z = B.Z;
	}
	Box.IsValid = 1;
}


//============================================================================//
//============================================================================//
//===                          FNodeGrid3 methods                          ===//
//============================================================================//
//============================================================================//


void FNodeGrid3::SetBaseGrid( FCollisionOctree* Base)
{
	GridSize = 1;
	Grid[0] = Base;

	BoxSize = Base->BoundingBox.Max - Base->BoundingBox.Min;
	Divider = FVector( _Reciprocal(BoxSize.X), _Reciprocal(BoxSize.Y), _Reciprocal(BoxSize.Z) );
	Substract = Base->BoundingBox.Min;
}

void FNodeGrid3::SetGrid( INT NewGridSize, const FBox& Bounds)
{
	check( NewGridSize > 0 );
	check( Bounds.IsValid);

	NewGridSize = 1 << NewGridSize;
	GridSize = NewGridSize;
//	INT GridMemSize = NewGridSize * NewGridSize * NewGridSize * 4;
//	appMemzero( &Grid[0], GridMemSize);

	Substract = Bounds.Min;
	BoxSize = (Bounds.Max - Bounds.Min) / (FLOAT)NewGridSize;
	Divider = FVector( _Reciprocal(BoxSize.X), _Reciprocal(BoxSize.Y), _Reciprocal(BoxSize.Z) );
}

void FNodeGrid3::Set( FCollisionOctree* In)
{
	FVector Pos( (In->CenterPoint.X - Substract.X) * Divider.X, (In->CenterPoint.Y - Substract.Y) * Divider.Y, (In->CenterPoint.Z - Substract.Z) * Divider.Z);
	INT X = Clamp( appFloor(Pos.X), 0, GridSize-1);
	INT Y = Clamp( appFloor(Pos.Y), 0, GridSize-1);
	INT Z = Clamp( appFloor(Pos.Z), 0, GridSize-1);
	In->XPos = X;		In->YPos = Y;		In->ZPos = Z;
	INT Point = X + Y * GridSize + Z * GridSize * GridSize;
	Grid[Point] = In;
}

void FNodeGrid3::Remove( FCollisionOctree* In)
{
	if ( !Grid ) //Something destroyed this first
		return;
	INT X = In->XPos;		INT Y = In->YPos;		INT Z = In->ZPos;
	INT Point = X + Y * GridSize + Z * GridSize * GridSize;
	Grid[Point] = NULL;
}

INT FNodeGrid3::GetIdx( const FVector& Pos)
{
	INT Point = Clamp( appFloor((Pos.X-Substract.X) * Divider.X), 0, GridSize-1);
	Point += Clamp( appFloor((Pos.Y-Substract.Y) * Divider.Y), 0, GridSize-1) * GridSize;
	Point += Clamp( appFloor((Pos.Y-Substract.Z) * Divider.Z), 0, GridSize-1) * GridSize * GridSize;
	return Point;
}

void FNodeGrid3::GetXYZ( const FVector& Pos, INT* XYZ)
{
	INT CLMax = GridSize-1;
	XYZ[0] = Clamp( appFloor((Pos.X-Substract.X) * Divider.X), 0, CLMax);
	XYZ[1] = Clamp( appFloor((Pos.Y-Substract.Y) * Divider.Y), 0, CLMax);
	XYZ[2] = Clamp( appFloor((Pos.Z-Substract.Z) * Divider.Y), 0, CLMax);
}

#endif
